package test.es.gob.jmulticard;

import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.PrivateKey;
import java.security.spec.RSAPrivateKeySpec;

import es.gob.jmulticard.HexUtils;

/** Imprime la codificaci&oacute;n PKCS#8 de la clave interna.
 * @author Tom&aacute;s Garc&iacute;a-Mer&aacute;s */
public final class CreatePrivateKey {

	private static final BigInteger MODULUS = new BigInteger(1, new byte[] {
		(byte) 0xba, (byte) 0x11, (byte) 0x38, (byte) 0x25, (byte) 0xf4, (byte) 0x17, (byte) 0x8a, (byte) 0x08, (byte) 0xc6, (byte) 0xaf,
		(byte) 0x31, (byte) 0xc1, (byte) 0x62, (byte) 0x12, (byte) 0x35, (byte) 0x62, (byte) 0x14, (byte) 0xd6, (byte) 0xce, (byte) 0x43,
		(byte) 0x6e, (byte) 0x2b, (byte) 0x61, (byte) 0x43, (byte) 0xde, (byte) 0xef, (byte) 0x0d, (byte) 0x08, (byte) 0x56, (byte) 0x2e,
		(byte) 0xfe, (byte) 0x97, (byte) 0x9d, (byte) 0x1e, (byte) 0xb8, (byte) 0xba, (byte) 0x6d, (byte) 0x2e, (byte) 0x95, (byte) 0x70,
		(byte) 0xcb, (byte) 0x4c, (byte) 0x3a, (byte) 0xe6, (byte) 0x93, (byte) 0xf1, (byte) 0xce, (byte) 0x6f, (byte) 0xaf, (byte) 0x34,
		(byte) 0x37, (byte) 0x2f, (byte) 0xe1, (byte) 0x4c, (byte) 0x99, (byte) 0x64, (byte) 0x16, (byte) 0xc2, (byte) 0xdd, (byte) 0xd7,
		(byte) 0xe4, (byte) 0xfd, (byte) 0xff, (byte) 0xd5, (byte) 0xfe, (byte) 0x4a, (byte) 0x6c, (byte) 0x3d, (byte) 0x38, (byte) 0xa9,
		(byte) 0x63, (byte) 0x67, (byte) 0xe8, (byte) 0xde, (byte) 0xcf, (byte) 0x45, (byte) 0xb1, (byte) 0x6f, (byte) 0x90, (byte) 0x04,
		(byte) 0x32, (byte) 0x2c, (byte) 0x59, (byte) 0x30, (byte) 0x37, (byte) 0x61, (byte) 0x67, (byte) 0xd8, (byte) 0xd7, (byte) 0x17,
		(byte) 0x3d, (byte) 0x79, (byte) 0x25, (byte) 0xea, (byte) 0x28, (byte) 0x6b, (byte) 0x30, (byte) 0x2b, (byte) 0x72, (byte) 0x60,
		(byte) 0xcf, (byte) 0x47, (byte) 0x99, (byte) 0xa8, (byte) 0x88, (byte) 0xb0, (byte) 0xb6, (byte) 0x59, (byte) 0x57, (byte) 0x3c,
		(byte) 0x1d, (byte) 0xe7, (byte) 0x07, (byte) 0xd9, (byte) 0x4e, (byte) 0x04, (byte) 0x06, (byte) 0xcc, (byte) 0xf0, (byte) 0xaa,
		(byte) 0xfa, (byte) 0x8e, (byte) 0x79, (byte) 0xff, (byte) 0xa1, (byte) 0x37, (byte) 0xb1, (byte) 0x77
    });

    private static final BigInteger PRIVATE_EXPONENT = new BigInteger(1, new byte[] {
		(byte) 0x23, (byte) 0x2b, (byte) 0x0c, (byte) 0x04, (byte) 0xc8, (byte) 0x6d, (byte) 0x06, (byte) 0x8f, (byte) 0x00, (byte) 0x16,
		(byte) 0xae, (byte) 0xfd, (byte) 0xd1, (byte) 0x2e, (byte) 0x85, (byte) 0xde, (byte) 0x82, (byte) 0xf7, (byte) 0xd2, (byte) 0xc6,
		(byte) 0x95, (byte) 0xf5, (byte) 0x27, (byte) 0x0e, (byte) 0x2f, (byte) 0xfd, (byte) 0xd8, (byte) 0xe5, (byte) 0xc1, (byte) 0x7c,
		(byte) 0x62, (byte) 0x6c, (byte) 0x7f, (byte) 0xab, (byte) 0x05, (byte) 0xd9, (byte) 0x5e, (byte) 0xb7, (byte) 0x96, (byte) 0x51,
		(byte) 0x5a, (byte) 0x6b, (byte) 0x8f, (byte) 0x1a, (byte) 0x73, (byte) 0xce, (byte) 0x19, (byte) 0x69, (byte) 0xac, (byte) 0xa0,
		(byte) 0xde, (byte) 0xe9, (byte) 0x72, (byte) 0xd9, (byte) 0x0a, (byte) 0x89, (byte) 0x0b, (byte) 0x70, (byte) 0x4b, (byte) 0x83,
		(byte) 0xf9, (byte) 0xe9, (byte) 0x34, (byte) 0x48, (byte) 0x88, (byte) 0xa1, (byte) 0x63, (byte) 0xf1, (byte) 0x4e, (byte) 0xe7,
		(byte) 0xc2, (byte) 0xbb, (byte) 0x02, (byte) 0x63, (byte) 0x40, (byte) 0x59, (byte) 0x12, (byte) 0x99, (byte) 0xaf, (byte) 0x1f,
		(byte) 0xc8, (byte) 0x44, (byte) 0x20, (byte) 0xd0, (byte) 0x0c, (byte) 0xb4, (byte) 0x42, (byte) 0x5e, (byte) 0xe2, (byte) 0xc1,
		(byte) 0x8a, (byte) 0xb8, (byte) 0x2f, (byte) 0x16, (byte) 0x04, (byte) 0x4f, (byte) 0x55, (byte) 0x02, (byte) 0xed, (byte) 0x23,
		(byte) 0x45, (byte) 0x7b, (byte) 0x4d, (byte) 0x83, (byte) 0x91, (byte) 0x29, (byte) 0x84, (byte) 0xef, (byte) 0x2e, (byte) 0x49,
		(byte) 0xa0, (byte) 0x4e, (byte) 0x1e, (byte) 0xfc, (byte) 0x40, (byte) 0x53, (byte) 0xc5, (byte) 0xf4, (byte) 0x81, (byte) 0x34,
		(byte) 0xe2, (byte) 0x57, (byte) 0x7d, (byte) 0xec, (byte) 0x19, (byte) 0xe3, (byte) 0xd8, (byte) 0x31
    });

    private CreatePrivateKey() {
    	// No instanciable
    }

	/** Imprime la codificaci&oacute;n PKCS#8 de la clave interna.
	 * @param args No se usa.
	 * @throws Exception En cualquier error. */
	public static void main(final String args[]) throws Exception {
        final RSAPrivateKeySpec privateSpec = new RSAPrivateKeySpec(MODULUS, PRIVATE_EXPONENT);
        final KeyFactory factory = KeyFactory.getInstance("RSA"); //$NON-NLS-1$
        final PrivateKey priv = factory.generatePrivate(privateSpec);
        System.out.println(
    		HexUtils.hexify(priv.getEncoded(), true)
		);
	}

}
